#!/usr/bin/perl

#       nakolesah_ru_parser.pl
#       
#       Copyright 2009 dimio (http://www.dimio.org, email dimio@dimio.org).
#       
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#       
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#       
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

require v5.8.8;
use warnings;
use strict;
use encoding 'utf8', STDOUT => 'cp866'; # для windows значение STDOUT => 'cp1251', современные ОС STDOUT => 'utf8', для ActivePerl STDOUT => 'cp866'
use Encode 'decode';
use LWP::UserAgent;
use HTTP::Cookies;
$|=1;
our $version = 1.0;
################ НАСТРОЙКИ ################
my $verbose = 1; # вывод сообщений о работе программы. 1 - выводить, 0 - не выводить
my $output_file = shift || $0; # имя выходного файла. можно задать при вызове скрипта
my $proxy_addr = ''; # http://127.0.0.1 для локального прокси например
my $proxy_port = ''; # 9050 например - tor
my $base_url = 'http://www.nakolesah.ru/'; # этот параметр без нужды менять не стоит
my $base_year = 1989; # тоже не нужно менять без необходимости - стартовый год для получения списка годов выпуска
################ КОНЕЦ НАСТРОЕК ################

my $start_time = join(':',(localtime)[2,1,0]);
print "Start: ". $start_time,"\n" if $verbose == 1;
$output_file =~ s/\.\w+/\.xml/;

my $browser = &UAInit($proxy_addr, $proxy_port); # инициализация юзер-агент для клиента
my $cars_brands_ref = &ParseTiresByAuto($browser,$base_url,$base_year); # НАБИГАЙ @ ОГРАБЛЯЙ
&PrintXML($output_file,$cars_brands_ref);
								
print "Started: ". $start_time,"\n" if $verbose == 1;
print "Stop: ". join(':',(localtime)[2,1,0]),"\n" if $verbose == 1;
print "Вывод перенаправлен в файл ". $output_file,"\n" if $verbose == 1;

exit 0;

sub ParseTiresByAuto($$$){ # создание списка машин

my ($browser,$base_url,$base_year) = @_;
my %car_brands;
my $tiresselect_url = $base_url.'select/tiresbyauto';

	## запрашиваем список всех марок машин
	my $response = $browser->get($tiresselect_url.'.aspx');
		die ("Connection error: ".$response->status_line,"\n") unless $response->is_success;
		print "Получение списка марок автомобилей: ".$response->status_line,"\n" if $verbose == 1;	
	my $source = decode('utf8', $response->content);
	redo if &WaitUpdating(\$source) == 1;
	
	## получаем список марок
	my $tmp = $1 if $source =~ m#<select name=.+id="ctl00_contentPlaceHolder_ddlVendors">(.+)</select>#is; # отсекаем всё лишнее до и после списка марок машин
	while ($tmp =~ m#<option value="([\w-]{36,})">([\wа-я]+)</option>#igs){ # разбор списка марок
		my $brand_key = $1;
		my $brand = &translate($2);
		
			# next if $brand !~ /Rover|FAW|Volkswagen|Ferrari|Jaguar|Smart|Suzuki|gaz|Bentley|Peugeot|Pontiac|Honda|Maybach|vaz|Infiniti|Buick|Subaru/i;
			# next if $brand !~ /Lancia|Opel|Daihatsu|Hummer|Kia|Fiat|Nissan|Saturn|Mini|Hyundai|Renault|Citroen|Lincoln|Chevrolet|Dodge/i;
			# next if $brand !~ /Chery|Mazda|Ford|uaz|Acura|Porsche|Lotus|Volvo|Toyota|Skoda|Cadillac|Scion|Saab|Mercury|Daewoo/i;
			# next if $brand !~ /Chrysler|BMW|Isuzu|MG|Mercedes|GMC|Seat|Maserati|Mitsubishi|Jeep|Lexus|Audi|Lifan|Geely/i;
		
		$car_brands{$brand} = +{};

		## получаем список моделей для каждой марки
			print "  Получение списка моделей для $brand\n" if $verbose == 1;
		$response = $browser->get($tiresselect_url.'/'.$brand.'.aspx');
			die ("Connection error: ".$response->status_line,"\n") unless $response->is_success;	
		my $source = decode('utf8', $response->content);
		redo if &WaitUpdating(\$source) == 1;
		
		my $tmp = $1 if $source =~ m#<select name=.+id="ctl00_contentPlaceHolder_ddlCars">(.+)</select>#is;
		while ($tmp =~ m#<option value="([\w-]{36,})">([\wа-я-]+)</option>#igs){ # разбор списка моделей	
			my $model_key = $1;
			my $car_model = &translate($2);
				$car_model = &TransformModel($brand,$car_model); # костыли для кривых названий (устранятся если перевести получение моделей на post-запрос). в функции печати вывода тоже подправить			
			$car_brands{$brand}{$car_model} = +{};

			## для каждой модели каждой марки получаем список годов выпуска
			my $years_html_code;
			my $year = $base_year;
				print "    Получение списка годов выпуска для $brand -> $car_model\n" if $verbose == 1;

			until (defined($years_html_code)){ ## пока не получена страница со списком всех годов выпуска - перебирать варианты
				$response = $browser->get($tiresselect_url.'/'.$brand.'/'.$car_model.'/'.$year.'.aspx');
					die ("Connection error: ".$response->status_line,"\n") unless $response->is_success;
				my $source = decode('utf8', $response->content);
				redo if &WaitUpdating(\$source) == 1;
				
				if ($source =~ m#<select name=.+id="ctl00_contentPlaceHolder_ddlYears">(.+)</select>#is){ # если в ответе сервера есть список годов - сохраним и обработаем его
					$years_html_code = $1;
					
					while ($years_html_code =~ m#<option.+?value="\d{4,}">(\d{4})</option>#igs){  # разбор списка годов выпуска
						my $car_year = $1;				
						$car_brands{$brand}{$car_model}{$car_year} = +{};

						## для каждого года получаем список всех модификаций
							print "      Получение списка модификаций для $brand -> $car_model -> $car_year\n" if $verbose == 1;
						$response = $browser->get($tiresselect_url.'/'.$brand.'/'.$car_model.'/'.$car_year.'.aspx');
							die ("Connection error: ".$response->status_line,"\n") unless $response->is_success;
						
						my $source = decode('utf8', $response->content);
						redo if &WaitUpdating(\$source) == 1;
						my $viewstate_key = $1 if $source =~ m#<input type="hidden" name="__VIEWSTATE" id="__VIEWSTATE" value="([\w=/+]+)" />#i; # ключик для передачи скрипту на сервере					
						my $tmp = $1 if $source =~ m#<select name=.+id="ctl00_contentPlaceHolder_ddlModifications">(.+)</select>#is;
						while ($tmp =~ m#<option value="([\w-]{36,})">([\wа-яА-Я\.\s\(\),-]+)</option>#igs){
							my $modifi_key = $1;
							my $car_modifi = $2;
							$car_brands{$brand}{$car_model}{$car_year}{$car_modifi} = +{};
							
							# для каждой модификации получаем список типоразмеров подходящих шин
								print "        Получение списка типоразмеров подходящих шин для $brand -> $car_model -> $car_year -> ".$car_modifi,"\n" if $verbose == 1;
							# только тут удаётся применить нормальный запрос к скрипту... выше по дереву в ответ вместо списка годов выпуска приходит пустота
							$response = $browser->post($tiresselect_url.'.aspx',
								[
								'__EVENTTARGET'								=>	'ctl00$contentPlaceHolder$ddlModifications',
								'__EVENTARGUMENT'							=>	'',
								'__LASTFOCUS'								=>	'',
								'__VIEWSTATE'								=>	$viewstate_key,
								'__ASYNCPOST'								=>	'true',
								'ctl00$contentPlaceHolder$scriptManager'	=>	'ctl00$contentPlaceHolder$upnlFilterAuto|ctl00$contentPlaceHolder$ddlModifications',
								'ctl00$contentPlaceHolder$cpExtenderFilter_ClientState'				=>	'false',
								'ctl00$contentPlaceHolder$ddlVendors'		=>	$brand_key,
								'ctl00$contentPlaceHolder$ddlCars'			=>	$model_key,
								'ctl00$contentPlaceHolder$ddlYears'			=>	$car_year,
								'ctl00$contentPlaceHolder$ddlModifications'	=>	$modifi_key,
								'ctl00$contentPlaceHolder$cpExtenderSendError_ClientState'			=>	'true',
								'ctl00$contentPlaceHolder$noBot$noBot_NoBotExtender_ClientState'	=>	int(rand(2000))+2000, # херь какая-то, но вдруг нужна? но_бот код...
								'ctl00$contentPlaceHolder$txtErrorAuto'		=>	'',
								'ctl00$contentPlaceHolder$txtErrorText'		=>	'',
								]);
								die ("Connection error: ".$response->status_line,"\n") unless $response->is_success;

							# делаем редирект на страницу со список всей требухи для нужной модификации... авторедирект почему-то не проходит... виню мудацкий .net aspx
							my $redir_url = $1 if $response->content =~ m#/([\wа-яА-Я\.\s\(\),%-]+)\|$#i; # url-encoding - не нужен тут, ссылка приходит уже закодированной (для этого % в шаблоне присутствует)
							
							$response = $browser->get($tiresselect_url.'/'.$brand.'/'.$car_model.'/'.$car_year.'/'.$redir_url);
								die ("Connection error: ".$response->status_line,"\n") unless $response->is_success;
							my $source = decode('utf8', $response->content);
							redo if &WaitUpdating(\$source) == 1;
							# $1 - шина = 1, диск = 2
							# $2 - OEM - штатная, Replacement - замена
							# $3 - мост. 1 - без разделения, 2 - передний, 3 - задний
							# $4 - типоразмер шины или диска (маркировка)						
							# получаем и сохраняем рекомендуемые и штатные типоразмеры шин и дисков
							while ($source =~ m#<td class="selectbyauto_applist_size.+?><a href=".+?'ctl00\$contentPlaceHolder\$repS(\d{1,})(\w+)\$?.+?\$ctl.(\d)'.+?">([\d\w\s/,^а-я]+)</a></td>#igs){
								my $axle = $3;
								$axle = 1 if $source !~ m#<td class="selectbyauto_applist_title">[передняя|задняя]\sось</td>#i;
								$car_brands{$brand}{$car_model}{$car_year}{$car_modifi}{'tires'}{$4}{$2} = $axle if $1 == 1;
								$car_brands{$brand}{$car_model}{$car_year}{$car_modifi}{'wheels'}{$4}{$2} = $axle if $1 == 2;
							}
							# получаем и сохраняем основные характеристики колёсных дисков
							while ($source =~ m#<strong>([a-zA-Z]{3}):</strong>.*?(\d\*\d{3}|\d{2},\d)#igs){
								$car_brands{$brand}{$car_model}{$car_year}{$car_modifi}{'wheels'}{$1} = $2;
							}	
						}
					}
				}
				$year++;
			}
		}
		print "\n\n" if $verbose == 1;
	}
	print "Готово\n\n" if $verbose == 1;

return \%car_brands;
}
sub PrintXML($$){
	my ($output_file,$car_brands) = @_;

	open (OUT,'>',$output_file) or die "Can't open XML output file: $!";
	print OUT '<?xml version="1.0" encoding="UTF-8"?>',"\n\n";

	foreach my $brand (keys %{ $car_brands }){
		print OUT '<brand name="'. $brand .'">', "\n";

		foreach my $model (keys %{ ${$car_brands}{$brand} }){
			if ($brand !~ m#Lincoln|Chery#i){
				print OUT "\t".'<model name="'. $model .'">', "\n";
			}
			elsif ($brand =~ m#Lincoln#i){
				my $tmp = $model;
				$tmp = 'Navigator' if $model eq 'navigaror_1';
				print OUT "\t".'<model name="'. $tmp .'">', "\n";
			}
			elsif ($brand =~ m#Chery#i){
				my $tmp = $model;
				$tmp = 'CrossEastar' if  $model eq 'c_eastar';
				$tmp = $1 if $model =~ /Chery_(kimo|qq\d?)/i;
				print OUT "\t".'<model name="'. $tmp .'">', "\n";
			}
			elsif ($brand =~ m#Fiat#i){
				my $tmp = $model;
				$tmp =~ s/%D1%81/c/ig;
				print OUT "\t".'<model name="'. $tmp .'">', "\n";
			}

			foreach my $year (keys %{ ${$car_brands}{$brand}{$model} }){
				print OUT "\t\t".'<year value="'. $year .'">', "\n";

				foreach my $modifi (keys %{ ${$car_brands}{$brand}{$model}{$year} }){
					print OUT "\t\t\t".'<modifi name="'. $modifi .'">', "\n";

					foreach my $type (keys %{ ${$car_brands}{$brand}{$model}{$year}{$modifi} }){
						print OUT "\t\t\t\t".'<type name="'. $type .'">', "\n";

						foreach my $label (sort {$b cmp $a} keys %{ ${$car_brands}{$brand}{$model}{$year}{$modifi}{$type} }){
							if ($label =~ /PCD|Dia/i){
								my $tmp = $label;
								$tmp =~ tr/A-Z/a-z/;
								print OUT "\t\t\t\t\t".'<'. $tmp .'>'. ${$car_brands}{$brand}{$model}{$year}{$modifi}{$type}{$label} .'</'. $tmp .'>', "\n";
							}
							elsif ($label !~ /PCD|Dia/i){
								print OUT "\t\t\t\t\t".'<label name="'. $label .'">', "\n";
								foreach my $completion (keys %{ ${$car_brands}{$brand}{$model}{$year}{$modifi}{$type}{$label} }){
									print OUT "\t\t\t\t\t\t".'<completion>'. $completion .'</completion>', "\n";

									print OUT "\t\t\t\t\t\t".'<axle> </axle>',"\n" if ${$car_brands}{$brand}{$model}{$year}{$modifi}{$type}{$label}{$completion} == 1;
									print OUT "\t\t\t\t\t\t".'<axle>front</axle>',"\n" if ${$car_brands}{$brand}{$model}{$year}{$modifi}{$type}{$label}{$completion} == 2;
									print OUT "\t\t\t\t\t\t".'<axle>rear</axle>',"\n" if ${$car_brands}{$brand}{$model}{$year}{$modifi}{$type}{$label}{$completion} == 3;
								}
								print OUT "\t\t\t\t\t".'</label>', "\n";
							}
						}
						print OUT "\t\t\t\t".'</type>', "\n";
					}
					print OUT "\t\t\t".'</modifi>', "\n";
				}
				print OUT "\t\t".'</year>', "\n";
			}
			print OUT "\t".'</model>', "\n";
		}
		print OUT '</brand>', "\n\n";
	}
	close (OUT);

}
sub WaitUpdating($){
	my $source = shift;
	my $sleep_time = 5;
	# если на сервере обновление базы - подождёмы
	if ($$source =~ m#<strong>Обновление данных!!!</strong><br />#){
		print 'На сервере обновление базы данных. Пауза '. $sleep_time .' секунд',"\n" if $verbose == 1;
		sleep($sleep_time); # спать N секунд
		return 1;
	}
	return 0;
}
sub UAInit($$){ # инициализация юзер-агента, прикидываемся браузером
my ($proxy_addr, $proxy_port) = @_;
my $cookies = HTTP::Cookies->new('file'=>'./cookies.lwp','autosave'=>0);
my $browser = LWP::UserAgent->new('agent' => 'Mozilla/5.0 (Windows; U; Windows NT 5.1; ru; rv:1.9.0.11) Gecko/2009060215 Firefox/3.0.11)',
								  'cookie_jar' => $cookies,
								  'requests_redirectable' => ['GET', 'POST']);
$browser->default_header('Accept' => 'text/html, application/xml;q=0.9, application/xhtml+xml, image/png, image/jpeg, image/gif, image/x-xbitmap, */*;q=0.1',
						'Accept-Charset' => 'utf-8; *;q=0.1',
						'Accept-Language' => 'ru,en-us;q=0.7,en;q=0.3',
						'Accept-Encoding' => 'deflate, gzip, x-gzip, identity, *;q=0',
						);
$browser->timeout(120); # при таймауте свыше 2 минут - закрывать соединение
 if ($proxy_addr and $proxy_port){
	$browser->proxy([qw(http https)], $proxy_addr . ':' . $proxy_port);
	print 'Proxy enabled: '. $proxy_addr . ':' . $proxy_port, "\n";
 }

 return $browser;
}
sub translate($){
	# Функция транслитерации кириллицы, сделанная для преобразования
	# заголовков сообщений в url-адреса, основана на коде nrg:
	# "Автор: nrg
	# Напиcано: Чтв Сен 13 00:04:14 SAMST 2007
	# Обратная связь: nrg@jabber.snc.ru, icq: 813-793."
	# dimio - добавлена очистка от мусора, запрет на транслитерацию составных слов и замена пробелов

 my $InLine = shift;
return $InLine if $InLine =~ m#^[a-zA-Z].+#;
	my %hs=('аА'=>'a' , 'бБ'=>'b'  , 'вВ'=>'v'  , 'гГ'=>'g', 'дД'=>'d' ,
			'еЕ'=>'e' , 'ёЁ'=>'jo' , 'жЖ'=>'zh' , 'зЗ'=>'z', 'иИ'=>'i' ,
			'йЙ'=>'j' , 'кК'=>'k'  , 'лЛ'=>'l'  , 'мМ'=>'m', 'нН'=>'n' ,
			'оО'=>'o' , 'пП'=>'p'  , 'рР'=>'r'  , 'сС'=>'s', 'тТ'=>'t' ,
			'уУ'=>'u' , 'фФ'=>'f'  , 'хХ'=>'h' 	, 'цЦ'=>'c', 'чЧ'=>'ch',
			'шШ'=>'sh', 'щЩ'=>'shh', 'ъЪ'=>''   , 'ыЫ'=>'y', 'ьЬ'=>''  ,
			'эЭ'=>'eh', 'юЮ'=>'ju' , 'яЯ'=>'ja' );

	pop@{([\map do{$InLine =~ s/[$_]/$hs{$_}/gi;}, keys %hs])}, $InLine;
	$InLine =~ s/\s/_/g;
	$InLine =~ s/[^\+\w\-]//g;

return $InLine;
}
sub TransformModel($$){
### из-за мудака, заполнившего базу разными названиями, нужно хуйнёй заниматься ((
my ($brand,$car_model) = @_;

	$car_model =~ s/-//g if $brand !~ /Saab|Jaguar|Nissan|Honda|Citroen|MG|Mercedes|Mazda|Ford/i;
	$car_model =~ s/[\-\+]/_/g if $brand !~ /Citroen/i;
	
	if ($brand =~ /Nissan/i){
		$car_model =~ s/Z/350z/i;
		$car_model =~ s/GT_R/GTR/i;
	}
	
	$car_model = 'navigaror_1' if $brand =~ m#Lincoln#i and $car_model eq 'Navigator';
	$car_model = 'Du%D1%81ato' if $brand =~ m#Fiat#i and $car_model =~ /duсato/i;
	
	if ($brand =~ /Chery/i){
		$car_model = 'c_eastar' if $car_model eq 'CrossEastar';
		$car_model = $brand .'_'. $car_model if $car_model =~ /kimo|qq\d?/i;
	}

return $car_model;
}
